function Map() {
  
  this.x = 6;
  this.y = 34;
  this.w = 684;
  this.h = 456;
  
  this.lots = [];
  
  this.buildings = {};
  this.buildingCounter = 0;
  
  this.employees = {};
  this.employeesCounter = 0;
  
  this.mouseMode = "select";
  
  this.buildTypeSelection;
  
  this.plannedTransports = [];
  
  this.money = 200;
  
  this.buildingCosts = {
    BarleyField  : 5,
    HopsField    : 5,
    WaterPump    : 10,
    Brewing      : 30,
    Fermentation : 25,
    Generator    : 10,
    Bottling     : 20,
    CrateMachine : 30
  };
  
  this.buildingPowers = {
    BarleyField  : 1,
    HopsField    : 1,
    WaterPump    : 5,
    Brewing      : 8,
    Fermentation : 2,
    Generator    : 0,
    Bottling     : 6,
    CrateMachine : 6
  };
  
  this.usedPower = 0;
  this.totalPower = 0;
  
  this.timeSinceLastCrate = 0;
  this.crateTimes = [];
  
  
  this.init = function() {
    for(var x = 0; x < 9; x++) {
      this.lots[x] = [];
      for(var y = 0; y < 6; y++) {
        this.lots[x][y] = -1;
      }
    }
    this.placeStartEmployees();
  };
  
  
  this.startBuildSelection = function(buildingType) {
    this.buildTypeSelection = buildingType;
    this.mouseMode = "build";
    mouse.registerUpArea("buildBuilding", this.x, this.y, this.w, this.h, function() {
      map.buildBuilding();
    });
  };
  
  
  this.buildBuilding = function() {
    var lot = this.getLot(mouse.x, mouse.y);
    var lotCoords = this.getLotCoords(lot.x, lot.y);
    if(this.lots[lot.x][lot.y] == -1) {
      if(this.usedPower < this.buildingPowers[this.buildTypeSelection]) {
        if(this.totalPower == 0) {
          panel.hint("Not enough power!", "Build a generator.");
        } else {
          panel.hint("Not enough power!", "Build more generators.");
        }
      } else if(this.money < this.buildingCosts[this.buildTypeSelection]) {
        panel.hint("Not enough money!", "Produce more crates.");
      } else {
        mouse.deleteUpArea("buildBuilding");
        this.mouseMode = "select";
        this.buildingCounter++;
        this.buildings[this.buildingCounter] = new Building();
        this.buildings[this.buildingCounter].init(this.buildTypeSelection, this.buildingCounter, lot.x, lot.y, 1, lotCoords.x, lotCoords.y);
        this.lots[lot.x][lot.y] = this.buildingCounter;
        this.money -= this.buildingCosts[this.buildTypeSelection];
        this.usedPower -= this.buildingPowers[this.buildTypeSelection];
        if(this.buildTypeSelection == "Generator") {
          this.usedPower += 40;
          this.totalPower += 40;
        }
      }
    }
  };
  
  
  this.getLot = function(x, y) {
    var lotX = Math.floor((x - this.x) / 76);
    if(lotX < 0) {
      lotX = 0;
    }
    if(lotX > 8) {
      lotX = 8;
    }
    var lotY = Math.floor((y - this.y) / 76);
    if(lotY < 0) {
      lotY = 0;
    }
    if(lotY > 5) {
      lotY = 5;
    }
    return { x : lotX, y : lotY };
  };
  
  
  this.getLotCoords = function(x, y) {
    return { x : map.x + (x * 76), y : map.y + (y * 76) };
  };
  
  
  this.placeStartEmployees = function() {
    for(var i = 0; i < 4; i++) {
      startX = rand((i * 2) + 1, (i * 2) + 2) * 76 + this.x;
      startY = rand(1, 5) * 76 + this.y;
      this.employeesCounter++;
      this.employees[this.employeesCounter] = new Employee();
      this.employees[this.employeesCounter].init();
      this.employees[this.employeesCounter].spawnAtStart(startX, startY);
    }
  };
  
  
  this.addEmployee = function() {
    if(this.money < 25) {
      panel.hint("Not enough money!", "Produce more crates.");
    } else {
      this.money -= 25;
      startX = rand(1, 8) * 76 + this.x;
      startY = 5 * 76 + this.y;
      this.employeesCounter++;
      this.employees[this.employeesCounter] = new Employee();
      this.employees[this.employeesCounter].init();
      this.employees[this.employeesCounter].spawn(startX, startY);
    }
  };
  
  
  this.update = function() {
    this.timeSinceLastCrate += 0.033;
    for(var x = 0; x < 9; x++) {
      for(var y = 0; y < 6; y++) {
        if(this.lots[x][y] != -1) {
          this.buildings[this.lots[x][y]].update();
        }
      }
    }
    this.checkTransport();
    for(var i in this.employees) {
      this.employees[i].update();
    }
  };
  
  
  this.checkTransport = function() {
    for(var i in this.buildings) {
      var building = this.buildings[i];
      if(building.type == "Brewing") {
        if(building.malt + building.maltOrdered < building.maltLimit) {
          for(var j in this.buildings) {
            if(this.buildings[j].type == "BarleyField" && this.buildings[j].malt - this.buildings[j].maltReserved > 0) {
              this.organizeTransport(j, i, "malt");
              break;
            }
          }
        }
        if(building.hops + building.hopsOrdered < building.hopsLimit) {
          for(var j in this.buildings) {
            if(this.buildings[j].type == "HopsField" && this.buildings[j].hops - this.buildings[j].hopsReserved > 0) {
              this.organizeTransport(j, i, "hops");
              break;
            }
          }
        }  
        if(building.water + building.waterOrdered < building.waterLimit) {
          for(var j in this.buildings) {
            if(this.buildings[j].type == "WaterPump" && this.buildings[j].water - this.buildings[j].waterReserved > 0) {
              this.organizeTransport(j, i, "water");
              break;
            }
          }
        }
        
        
      } else if(building.type == "Fermentation") {
        
        if(building.wort + building.wortOrdered < building.wortLimit) {
          for(var j in this.buildings) {
            if(this.buildings[j].type == "Brewing" && this.buildings[j].wort - this.buildings[j].wortReserved > 0) {
              this.organizeTransport(j, i, "wort");
              break;
            }
          }
        }
        
        
      } else if(building.type == "Bottling") {
        if(building.kegs + building.kegsOrdered < building.kegsLimit) {
          for(var j in this.buildings) {
            if(this.buildings[j].type == "Fermentation" && this.buildings[j].kegs - this.buildings[j].kegsReserved > 0) {
              this.organizeTransport(j, i, "keg");
              break;
            }
          }
        }
        
        
      } else if(building.type == "CrateMachine") {
        
        if(building.bottles + building.bottlesOrdered < building.bottlesLimit) {
          for(var j in this.buildings) {
            if(this.buildings[j].type == "Bottling" && this.buildings[j].bottles - this.buildings[j].bottlesReserved > 0) {
              this.organizeTransport(j, i, "bottle");
              break;
            }
          }
        }
        
        if(building.crates > 0) {
          this.crateTimes.push(this.timeSinceLastCrate);
          this.timeSinceLastCrate = 0;
          if(this.crateTimes.length > 5) {
            this.crateTimes.shift();
          }
          building.crates--;
          this.money += 60;
        }
      }
    }
  };
  
  
  this.organizeTransport = function(startBuilding, endBuilding, ressource) {
    if(ressource == "malt") {
      this.buildings[startBuilding].maltReserved++;
      this.buildings[endBuilding].maltOrdered++;
    } else if(ressource == "hops") {
      this.buildings[startBuilding].hopsReserved++;
      this.buildings[endBuilding].hopsOrdered++;
    } else if(ressource == "water") {
      this.buildings[startBuilding].waterReserved++;
      this.buildings[endBuilding].waterOrdered++;
    } else if(ressource == "wort") {
      this.buildings[startBuilding].wortReserved++;
      this.buildings[endBuilding].wortOrdered++;
    } else if(ressource == "keg") {
      this.buildings[startBuilding].kegsReserved++;
      this.buildings[endBuilding].kegsOrdered++;
    } else if(ressource == "bottle") {
      this.buildings[startBuilding].bottlesReserved++;
      this.buildings[endBuilding].bottlesOrdered++;
    } else if(ressource == "crate") {
      this.buildings[startBuilding].cratesReserved++;
      this.buildings[endBuilding].cratesOrdered++;
    }
    this.plannedTransports.push({start: startBuilding, end: endBuilding, ressource : ressource});
  };
  
  
  this.hasNextTransport = function() {
    return this.plannedTransports.length > 0;
  };
  
  
  this.getNextTransport = function() {
    return this.plannedTransports.shift();
  };
  
  
  this.getTwoCornersOfBuilding = function(buildingId) {
    var building = this.buildings[buildingId];
    var lotCoords = this.getLotCoords(building.x, building.y);
    return { x1 : lotCoords.x, y1 : lotCoords.y + 76, x2 : lotCoords.x + 76, y2 : lotCoords.y + 76 };
  };
  
  
  this.getLoadingPointOfBuilding = function(buildingId) {
    var building = this.buildings[buildingId];
    var lotCoords = this.getLotCoords(building.x, building.y);
    return { x : lotCoords.x + 62, y : lotCoords.y + 62 };
  };
  
  
  this.getDropoffPointOfBuilding = function(buildingId) {
    var building = this.buildings[buildingId];
    var lotCoords = this.getLotCoords(building.x, building.y);
    return { x : lotCoords.x + 14, y : lotCoords.y + 62 };
  };
  
  
  this.draw = function() {
    
    img.draw("Map", this.x, this.y);
    
    for(var x = 0; x < 9; x++) {
      for(var y = 0; y < 6; y++) {
        if(this.lots[x][y] != -1) {
          this.buildings[this.lots[x][y]].draw();
        }
      }
    }
    for(var i in this.employees) {
      this.employees[i].draw();
    }
    for(var i in this.buildings) {
      this.buildings[i].drawParticles();
    }
    if(mouse.isOver(this.x, this.y, this.w, this.h)) {
      if(this.mouseMode == "build") {
        var lot = this.getLot(mouse.x, mouse.y);
        var lotCoords = this.getLotCoords(lot.x, lot.y);
        if(this.lots[lot.x][lot.y] == -1) {
          c.globalAlpha = 0.4;
          c.fillStyle = "#0c0";
          c.fillRect(lotCoords.x, lotCoords.y, 76, 76);
          c.globalAlpha = 0.6;
          img.drawSprite(this.buildTypeSelection, lotCoords.x, lotCoords.y, 76, 76, 0, 0);
          c.globalAlpha = 1.0;
        } else {
          c.globalAlpha = 0.4;
          c.fillStyle = "#e00";
          c.fillRect(lotCoords.x, lotCoords.y, 76, 76);
          c.globalAlpha = 1.0;
        }
      }
    }
  };
  
};